function collision_trigger()

	function calculate_distance(x1, y1, x2, y2)
		return math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
	end

	local entity_id = GetUpdatedEntityID()
	local x, y, rot = EntityGetTransform(entity_id)
	
	local components = EntityGetComponent(entity_id, "VariableStorageComponent")
    if components ~= nil then
        for _, comp_id in pairs(components) do
            local var_name = ComponentGetValue(comp_id, "name")
            if var_name == "projectiles" then
                projectiles_var_comp = comp_id
            end
        end
    end
	
	--Getting all projectiles and their explosions to simulate digging
	if projectiles_var_comp ~= nil and projectiles_var_comp > 0 then

		function split_string(input_str, delimiter)
			local result = {}
			for match in (input_str .. delimiter):gmatch("(.-)" .. delimiter) do
				table.insert(result, match)
			end
			return result
		end
		
		function decode_stored_coords(val)
			local out = {}
			local stored_coords = split_string(val, ",")
			for i=1, #stored_coords, 4 do
				table.insert(out, {id = stored_coords[i], radius = tonumber(stored_coords[i+1]), x = tonumber(stored_coords[i+2]), y = tonumber(stored_coords[i+3])})
			end
			return out
		end
		
		function encode_coords_to_store(t)
			local result = {}
			for _, data in ipairs(t) do
				table.insert(result, data.id)
				table.insert(result, tostring(data.radius))
				table.insert(result, tostring(data.x))
				table.insert(result, tostring(data.y))
			end
			return table.concat(result, ",")
		end
		
		-- Retrieve previous projectiles
		local val = ComponentGetValue2(projectiles_var_comp, "value_string")
		local previous_projectiles = decode_stored_coords(val)
		
		local projectiles = {}
		local entities = EntityGetInRadius(x, y, 40)
		local max_projectiles = 1
		
		-- Calculate priority score and collect relevant projectiles
		for _, entity in ipairs(entities) do
			local projectile_component = EntityGetFirstComponent(entity, "ProjectileComponent")
			if projectile_component ~= nil and projectile_component > 0 then
				local on_death_explodes = ComponentGetValue2(projectile_component, "on_death_explode")
				local on_lifetime_out_explodes = ComponentGetValue2(projectile_component, "on_lifetime_out_explode")
				if on_death_explodes == true or on_lifetime_out_explodes == true then
					local explosion_radius = ComponentObjectGetValue2(projectile_component, "config_explosion", "explosion_radius")
					local px, py = EntityGetTransform(entity)
					local distance = math.sqrt((px - x) ^ 2 + (py - y) ^ 2)
					local priority_score = distance - explosion_radius
					table.insert(projectiles, {id = entity, radius = explosion_radius, x = px, y = py, score = priority_score})
				end
			end
		end

		-- Sort projectiles based on priority score (lower score is higher priority)
		table.sort(projectiles, function(a, b) return a.score < b.score end)

		-- Keep only the top projectiles up to the max limit
		local prioritized_projectiles = {}
		for i = 1, math.min(max_projectiles, #projectiles) do
			table.insert(prioritized_projectiles, projectiles[i])
		end

		-- Function to check if explosion overlaps with plant corners
		function check_explosion_overlap(explosion_x, explosion_y, explosion_radius, offsets)
			for i=1, #offsets, 2 do
				local offset_x = offsets[i]
				local offset_y = offsets[i + 1]
				local pos_x = x - offset_x
				local pos_y = y - offset_y
				
				-- GameCreateSpriteForXFrames("mods/particle_life/files/plants/debug_indicator7.png", pos_x, pos_y, true, 0, 0, 1, true)

				local distance = calculate_distance(pos_x, pos_y, explosion_x, explosion_y)
				if distance <= explosion_radius then
					return true
				end
			end

			return false
		end
		
		-- Check for expired projectiles
		local sprite_offsets = {}
		local sprite_comps = EntityGetComponent(entity_id, "SpriteComponent") or {}
		for i, comp in ipairs(sprite_comps) do
			local offset_x = ComponentGetValue2(comp, "offset_x")
			local offset_y = ComponentGetValue2(comp, "offset_y")
			table.insert(sprite_offsets, offset_x)
			table.insert(sprite_offsets, offset_y)
		end
		for _, prev_proj in ipairs(previous_projectiles) do
			if not EntityGetIsAlive(tonumber(prev_proj.id)) then
				if prev_proj.radius ~= nil then
					-- GamePrint("Yes, explosion radius was: " .. tostring(prev_proj.radius).. " " .. tostring(prev_proj.x).. " " .. tostring(prev_proj.y))
					
					local overlap = check_explosion_overlap(tonumber(prev_proj.x), tonumber(prev_proj.y), tonumber(prev_proj.radius), sprite_offsets)
					if overlap == true then
						EntityKill(entity_id)
					end
				end
			end
		end
		
		-- Update the VariableStorageComponent with the new list of active projectiles
		local encode_string = encode_coords_to_store(prioritized_projectiles)
		ComponentSetValue2(projectiles_var_comp, "value_string", encode_string)
	end
	
	local times = ComponentGetValue2(GetUpdatedComponentID(), "mTimesExecuted")
	
	local is_offscreen = false

	local cbx, cby, cbw, cbh = GameGetCameraBounds()
	if ( ( x < cbx ) or ( x > cbx + cbw ) or ( y < cby ) or ( y > cby + cbh ) ) then
		is_offscreen = true
	end
	
	local is_active_component = EntityGetFirstComponentIncludingDisabled(entity_id, "VariableStorageComponent")
	if ( is_active_component ~= nil ) and ( is_active_component > 0 ) then
		if ( ComponentGetValue2(is_active_component, "value_bool") == true ) then
			--GamePrint("Activating")
			

			local components = EntityGetComponent(entity_id, "VariableStorageComponent")
			if components ~= nil then
				for _, comp_id in pairs(components) do
					local var_name = ComponentGetValue(comp_id, "name")
					if var_name == "current_position" then
						current_position_var_comp = comp_id
					end
					if var_name == "previous_positions" then
						previous_positions_var_comp = comp_id
					end
					if var_name == "init" then
						init_var_comp = comp_id
					end
				end
			end
			
			local ox_raw = ComponentGetValue2(current_position_var_comp, "value_float")
			local oy_raw = ComponentGetValue2(current_position_var_comp, "value_string")

			if ComponentGetValue2(init_var_comp, "value_bool") == false then
				ComponentSetValue2(init_var_comp, "value_bool", true)

				ComponentSetValue2(current_position_var_comp, "value_float", x)
				ComponentSetValue2(current_position_var_comp, "value_string", tostring(y))
				
				ComponentSetValue2(previous_positions_var_comp, "value_string", tostring(x) .. "," .. tostring(y))
				ox = x
				oy = y
			else
				ox = ox_raw
				oy = tonumber(oy_raw)
			end

			local previous_positions = ComponentGetValue2(previous_positions_var_comp, "value_string")

			SetRandomSeed(GameGetFrameNum(), x + y + entity_id)

			function load_vegetation_pixel(image_file2, px, py)
				local pixelcomp = EntityAddComponent2(entity_id, "SpriteComponent", {
					image_file = image_file2,
					offset_x = px,
					
					--  x - ox -1, y - oy
					
					offset_y = py,
					update_transform_rotation = false,
					has_special_scale = true,
					special_scale_x = 1,
					special_scale_y = 1
				})
				ComponentAddTag( pixelcomp, "enabled_in_world" )
				
				-- print(px)
				-- print(py)
				-- print(image_file2)
			end

			-- Function to encode the coordinates into a string
			function encode_coordinates(coord_array)
				local encoded_string = ""
				for i, coord in ipairs(coord_array) do
					if i > 1 then
						encoded_string = encoded_string .. ","
					end
					encoded_string = encoded_string .. tostring(coord)
				end
				return encoded_string
			end

			-- Function to decode the string of coordinates into an array
			function decode_coordinates(encoded_string)
				if encoded_string == nil then
					return {}
				end
				local coord_array = {}
				for coord in string.gmatch(encoded_string, "([^,]+)") do
					table.insert(coord_array, tonumber(coord))
				end
				return coord_array
			end

			-- Function to check if a new position overlaps with existing coordinates
			function is_position_overlapping(new_x, new_y, previous_positions)
				for i = 1, #previous_positions, 2 do
					local x2 = previous_positions[i]
					local y2 = previous_positions[i + 1]
					if ( math.abs(x2 - new_x) < 0.001 ) and ( math.abs(y2 - new_y) < 0.001 ) then
						return true
					end
				end
				return false
			end

			-- -- Function to calculate radial position
			-- function calculate_radial_position(center_x, center_y, radius, angle)
				-- local x = center_x + radius * math.cos(angle)
				-- local y = center_y + radius * math.sin(angle)
				-- return x, y
			-- end

			-- -- Function to place flower pixels radially around the central flower
			-- function place_flower_pixels(center_x, center_y, num_petals, petal_length, flower_texture)
				-- for i = 1, num_petals do
					-- local angle = (i - 1) * (2 * math.pi / num_petals) -- Calculate angle for each petal
					-- local petal_x, petal_y = calculate_radial_position(center_x, center_y, petal_length, angle)
					-- -- Place the flower pixel at the calculated position
					-- move_and_place_pixel(center_x, center_y, petal_x - center_x, petal_y - center_y, current_position_var_comp, previous_positions_var_comp, false, flower_texture)
				-- end
			-- end

			function move_and_place_pixel(oldx, oldy, nx, ny, var_comp1, var_comp2, image_file)
				local new_x = oldx - nx
				local new_y = oldy - ny
				local coord_string = ComponentGetValue2(var_comp2, "value_string")
				local decoded_coordinates = decode_coordinates(coord_string)
				
				local is_overlapping_others = false
				local other_plants = EntityGetInRadiusWithTag( x, y, 50, "prop" )
				if #other_plants > 0 then
					for i,plant in ipairs(other_plants) do
					
						if EntityGetName( plant ) == "particle_life_plant" then
							local components = EntityGetComponent(plant, "VariableStorageComponent")
							if components ~= nil then
								for _, comp_id in pairs(components) do
									local var_name = ComponentGetValue(comp_id, "name")
									if var_name == "current_position" then
										current_position2_var_comp = comp_id
									end
									if var_name == "previous_positions" then
										previous_positions2_var_comp = comp_id
									end
								end
							end
							local plant_coord_string = ComponentGetValue2(previous_positions2_var_comp, "value_string")
							local decoded_coordinates2 = decode_coordinates(plant_coord_string)
							
							local pox_raw = ComponentGetValue2(current_position2_var_comp, "value_float")
							local poy_raw = ComponentGetValue2(current_position2_var_comp, "value_string")

							local pox = pox_raw
							local poy = tonumber(poy_raw)
							
							table.insert(decoded_coordinates2, pox)
							table.insert(decoded_coordinates2, poy)
							
							overlapping_others = is_position_overlapping(new_x, new_y, decoded_coordinates2)
							if overlapping_others == true then
								is_overlapping_others = true
							end
						end
					end

				end
				-- if is_overlapping_others == true then
					-- GamePrint("I'm overlapping others")
					-- GameCreateSpriteForXFrames( "mods/particle_life/files/plants/debug_indicator3.png", new_x, new_y, true, 0, 0, 99999, true )
				-- end
				if ( is_position_overlapping(new_x, new_y, decoded_coordinates) == false ) and ( is_overlapping_others == false ) then
					load_vegetation_pixel(image_file, x - oldx, y - oldy)
					
					ComponentSetValue2(var_comp1, "value_float", new_x)
					ComponentSetValue2(var_comp1, "value_string", tostring(new_y))

					table.insert(decoded_coordinates, new_x)
					table.insert(decoded_coordinates, new_y)
					local new_coord_string = encode_coordinates(decoded_coordinates)
					ComponentSetValue2(var_comp2, "value_string", new_coord_string)
				end
				-- GameCreateSpriteForXFrames( "data/debug/sui_close_button.png", new_x, new_y, true, 0, 0, 1, true )
				-- print(new_coord_string)
				-- print(new_x)
				-- print(new_y)
				-- print(image_file)
			end

			local random_number = Random(1, 8)
			local random_number2 = Random(1,10)

			local no_movement = false

			local did_hit_left = Raytrace(ox - 1, oy , ox - 1, oy)
			local did_hit_right = Raytrace(ox + 1, oy, ox + 1, oy)
			local did_hit_left_top = Raytrace(ox - 1, oy - 1 , ox - 1, oy -1)
			local did_hit_right_top = Raytrace(ox + 1, oy - 1 , ox + 1, oy -1)
			local did_hit_left_bottom = Raytrace(ox - 1, oy + 1 , ox - 1, oy + 1)
			local did_hit_right_bottom = Raytrace(ox + 1, oy + 1 , ox + 1, oy + 1)
			local did_hit_bottom = Raytrace(ox, oy + 1, ox, oy + 1)
			local did_hit_top = Raytrace(ox, oy - 1, ox, oy - 1)

			local textures = {"mods/particle_life/files/plants/plant_texture1.png",
			"mods/particle_life/files/plants/plant_texture2.png",
			"mods/particle_life/files/plants/plant_texture3.png"
			}
			local random_index = Random(1, #textures)
			local plant_texture = textures[random_index]

			local ftextures = {"mods/particle_life/files/plants/plant_flower_texture1",
			"mods/particle_life/files/plants/plant_flower_texture2",
			"mods/particle_life/files/plants/plant_flower_texture3"
			}
			local random_index2 = Random(1, #ftextures)
			local flower_texture = ftextures[random_index2]

			local flower_texture_middle = flower_texture .. "_center.png"

			flower_texture = flower_texture .. ".png"

			if ( random_number == 1 ) and not did_hit_right then
				move_and_place_pixel(ox, oy, -1, 0, current_position_var_comp, previous_positions_var_comp, plant_texture)
				
			elseif ( random_number == 2 ) and not did_hit_left then
				move_and_place_pixel(ox, oy, 1, 0, current_position_var_comp, previous_positions_var_comp, plant_texture)
				
			elseif ( random_number == 3 ) and not did_hit_right_bottom then
				load_vegetation_pixel(plant_texture, x - ox -1, y - oy)
				move_and_place_pixel(ox, oy, -1, 0, current_position_var_comp, previous_positions_var_comp, plant_texture)
				move_and_place_pixel(ox, oy, -1, -1, current_position_var_comp, previous_positions_var_comp, plant_texture)
				
			elseif ( random_number == 4 ) and not did_hit_left_bottom then
				load_vegetation_pixel(plant_texture, x - ox +1, y - oy)
				move_and_place_pixel(ox, oy, 1, 0, current_position_var_comp, previous_positions_var_comp, plant_texture)
				move_and_place_pixel(ox, oy, 1, -1, current_position_var_comp, previous_positions_var_comp, plant_texture)

			elseif ( random_number == 5 ) and not did_hit_right_top then
				load_vegetation_pixel(plant_texture, x - ox -1, y - oy)
				move_and_place_pixel(ox, oy, -1, 0, current_position_var_comp, previous_positions_var_comp, plant_texture)
				move_and_place_pixel(ox, oy, -1, 1, current_position_var_comp, previous_positions_var_comp, plant_texture)
				
			elseif ( random_number == 6 ) and not did_hit_left_top then
				load_vegetation_pixel(plant_texture, x - ox +1, y - oy)
				move_and_place_pixel(ox, oy, 1, 0, current_position_var_comp, previous_positions_var_comp, plant_texture)
				move_and_place_pixel(ox, oy, 1, 1, current_position_var_comp, previous_positions_var_comp, plant_texture)

			elseif ( random_number == 7 ) and not did_hit_bottom then
				move_and_place_pixel(ox, oy, 0, -1, current_position_var_comp, previous_positions_var_comp, plant_texture)
				
			elseif ( random_number == 8 ) and not did_hit_top then
				move_and_place_pixel(ox, oy, 0, 1, current_position_var_comp, previous_positions_var_comp, plant_texture)
			else
				
				local end_growth = false
				
				if ( did_hit_left and did_hit_right and did_hit_bottom and did_hit_top and did_hit_right_bottom and did_hit_right_top and did_hit_left_bottom and did_hit_left_top ) then
				
					end_growth = true
					
				elseif random_number2 < 2 then
					load_vegetation_pixel(flower_texture_middle, x - ox, y - oy)
					load_vegetation_pixel(flower_texture, x - ox -1, y - oy)
					load_vegetation_pixel(flower_texture, x - ox +1, y - oy)
					load_vegetation_pixel(flower_texture, x - ox, y - oy - 1)
					load_vegetation_pixel(flower_texture, x - ox, y - oy + 1)
					end_growth = true
				end
				if end_growth == true then
					ComponentSetValue2(is_active_component, "value_bool", false)
				end
			end


			if ( times ~= nil ) and ( times > 100 ) then
				if random_number2 < 5 then
					load_vegetation_pixel(flower_texture_middle, x - ox, y - oy)
					load_vegetation_pixel(flower_texture, x - ox -1, y - oy)
					load_vegetation_pixel(flower_texture, x - ox +1, y - oy)
					load_vegetation_pixel(flower_texture, x - ox, y - oy - 1)
					load_vegetation_pixel(flower_texture, x - ox, y - oy + 1)
				end
				ComponentSetValue2(is_active_component, "value_bool", false)
			end
			
		elseif ( ComponentGetValue2(is_active_component, "value_bool") == false ) and ( is_offscreen == false ) then
			-- local imprint_var_comp_old = 0
			local components = EntityGetComponent(entity_id, "VariableStorageComponent")
			if components ~= nil then
				for _, comp_id in pairs(components) do
					local var_name = ComponentGetValue(comp_id, "name")
					if var_name == "previous_positions" then
						previous_positions_var_comp = comp_id
					end
					-- if var_name == "imprint" then
						-- imprint_var_comp_old = comp_id
					-- end
				end
			end
			
			-- if imprint_var_comp_old == 0 then
			
				-- -- Initialize an imprint table to store initial positions and textures
				-- local imprint_table = {}

				-- -- Iterate through all PixelSpriteComponents and capture initial positions and textures
				-- local pixel_components = EntityGetComponentIncludingDisabled(entity_id, "PixelSpriteComponent")
				-- if (pixel_components ~= nil) and (#pixel_components > 0) then
					-- for _, pixel_comp in ipairs(pixel_components) do
						-- local anchor_x = ComponentGetValue2(pixel_comp, "anchor_x")
						-- local anchor_y = ComponentGetValue2(pixel_comp, "anchor_y")
						-- local texture = ComponentGetValue2(pixel_comp, "image_file")

						-- -- Store initial positions and texture in imprint table
						-- table.insert(imprint_table, texture)
						-- table.insert(imprint_table, anchor_x)
						-- table.insert(imprint_table, anchor_y)
					-- end
				-- end

				-- -- Convert imprint table to string
				-- local imprint_str = table.concat(imprint_table, ",")
				
				-- EntityAddComponent2(entity_id, "VariableStorageComponent", {
					-- name = "imprint",
					-- value_string = imprint_str
				-- })
			-- end
			
			local players = EntityGetInRadiusWithTag(x, y, 50, "player_unit")
			local player_id = 0
			if #players > 0 then
				player_id = players[1]
				local px, py = EntityGetTransform(player_id)
				local player_shooter_component = EntityGetFirstComponent(player_id, "PlatformShooterPlayerComponent")
				local controls_component = EntityGetFirstComponent(player_id, "ControlsComponent")
				local character_data_component = EntityGetFirstComponent(player_id, "CharacterDataComponent")
				if ( controls_component ~= nil ) and ( controls_component > 0 ) then
					local is_eating = false
					local button_down = ComponentGetValue2(controls_component, "mButtonDownDown")
					if ( character_data_component ~= nil ) and ( character_data_component > 0 ) and ( button_down == true ) then
						local is_on_ground = ComponentGetValue2(character_data_component, "is_on_ground")
						if is_on_ground == true then
							is_eating = true
						end
					end
					if ( player_shooter_component ~= nil ) and ( player_shooter_component > 0 ) and ( is_eating == true ) then
						local x_min, y_min = ComponentGetValue2(player_shooter_component, "eating_area_min")
						local x_max, y_max = ComponentGetValue2(player_shooter_component, "eating_area_max")
						
						y_min = y_min - 5

						-- Decode the string of previous positions into an array
						local previous_positions_str = ComponentGetValue2(previous_positions_var_comp, "value_string")
						local previous_positions = decode_coordinates(previous_positions_str)
						
						local pixel_sprite_components = EntityGetComponentIncludingDisabled(entity_id, "SpriteComponent") or {}
						local first_comp = 0

						-- Iterate through the previous positions
						for i = 1, #previous_positions, 2 do
							local pos_x = previous_positions[i]
							local pos_y = previous_positions[i + 1]

							-- Check if the position is within the eating area
							if pos_x >= (px + x_min) and pos_x <= (px + x_max) and pos_y >= (py + y_min) and pos_y <= (py + y_max) then
								
								-- Remove pixels above the eating area
								for j = 1, i - 2, 2 do
									remove_x = previous_positions[j]
									remove_y = previous_positions[j + 1]
									
									if remove_x and remove_y then
										local first_index = 0
										if ( pixel_sprite_components ~= nil ) and ( #pixel_sprite_components > 0 ) then
											for index, comp in ipairs(pixel_sprite_components) do
												local anchorx = ComponentGetValue2(comp, "offset_x")
												local anchory = ComponentGetValue2(comp, "offset_y")
												-- local eatx = (x - remove_x)
												-- local eaty = (y - remove_y)
												
												-- GamePrint(tostring(eatx) .. " , " .. tostring(eaty))
												local arxdist = anchorx - (x - remove_x)
												local arydist = anchory - (y - remove_y)
												
												if ( math.abs(arxdist) < 0.001 ) and ( math.abs(arydist) < 0.001 ) then
												-- if ( anchorx == eatx ) and ( anchory == eaty ) then
													-- EntityRemoveComponent(entity_id, comp)
													first_comp = comp
													break
												end
											end
										end
										-- Remove the corresponding position from the array
										table.remove(previous_positions, j)
										table.remove(previous_positions, j)
										j = j - 2 -- Adjust the index after removal
									end
								end
								-- Update the previous positions string
								local new_previous_positions_str = encode_coordinates(previous_positions)
								ComponentSetValue2(previous_positions_var_comp, "value_string", new_previous_positions_str)
								break -- Exit the loop after processing the eating area
							end
						end
						
						if first_comp ~= nil and first_comp > 0 then
							for i = #pixel_sprite_components, 1, -1 do
								local comp_id = pixel_sprite_components[i]
								if comp_id ~= first_comp then
									local anchorx = ComponentGetValue2(comp_id, "offset_x")
									local anchory = ComponentGetValue2(comp_id, "offset_y")
									EntityLoad( "mods/particle_life/files/plants/plant_corpse.xml", x - anchorx , y - anchory )
									-- EntityIngestMaterial( player_id, CellFactory_GetType( "grass" ), 0.2 )
									EntityRemoveComponent(entity_id, comp_id)
								else
									break
								end
							end
						end
					end
				end
			end

		end
	end
	
	local did_hit_root_up = Raytrace(x, y, x, y - 1)
	local did_hit_root = Raytrace(x, y, x, y + 1)
	local did_hit_root_right = Raytrace(x, y, x + 1, y)
	local did_hit_root_left = Raytrace(x, y, x - 1, y)
	if ( not did_hit_root_up ) and ( not did_hit_root ) and ( not did_hit_root_right ) and ( not did_hit_root_left ) then
		EntityKill(entity_id)
	end
	local this_lua = GetUpdatedComponentID()
	if ( this_lua ~= nil ) and ( this_lua > 0 ) then
		if is_offscreen == false then
			-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", true)
			ComponentSetValue2(this_lua, "limit_to_every_n_frame", 0)
			ComponentSetValue2(this_lua, "limit_all_callbacks", false)
		else
			-- EntitySetComponentsWithTagEnabled(entity_id, "enabled_in_world", false)
			ComponentSetValue2(this_lua, "limit_to_every_n_frame", 120)
			ComponentSetValue2(this_lua, "limit_all_callbacks", true)
		end
	end
end